--[[
Name: IHUD_Threat
Version: 1.2
Author: Caryna/Turalyon EU (Alliance)
Description: adds a threat bar to IceHUD
]]

local AceOO = AceLibrary("AceOO-2.0")
local Threat = AceLibrary("Threat-1.0")

local IHUD_Threat = AceOO.Class(IceUnitBar)

IHUD_Threat.prototype.color = nil
IHUD_Threat.aggroBar = nil
IHUD_Threat.aggroBarMulti = nil
IHUD_Threat.debug = false

-- debug print stuff
local function Print(x)
	if ( IHUD_Threat.debug ) then
		DEFAULT_CHAT_FRAME:AddMessage(x)
	end
end

-- constructor
function IHUD_Threat.prototype:init()
	IHUD_Threat.super.prototype.init(self, "Threat", "target")

	self:SetDefaultColor("ThreatLow", 102, 204, 51)
	self:SetDefaultColor("ThreatMedium", 0, 204, 204)
	self:SetDefaultColor("ThreatHigh", 204, 0, 153)
	self:SetDefaultColor("ThreatDanger", 255, 0, 0)
	self:SetDefaultColor("ThreatCustom", 255, 255, 0)
	
	self:OnCoreLoad()
end

-- default settings
function IHUD_Threat.prototype:GetDefaultSettings()
	local settings = IHUD_Threat.super.prototype.GetDefaultSettings(self)
	settings["side"] = IceCore.Side.Left
	settings["offset"] = 3
	settings["debug"] = false
	return settings
end

-- options stuff
function IHUD_Threat.prototype:GetOptions()
	local opts = IHUD_Threat.super.prototype.GetOptions(self)
	
	opts["debug"] = {
		type = "toggle",
		name = "Debug mode",
		desc = "Display debug info",
		get = function()
			return self.moduleSettings.debug
		end,
		set = function(value)
			self.moduleSettings.debug = value
			IHUD_Threat.debug = value
			self:Update(self.unit)
		end,
		disabled = function()
			return not self.moduleSettings.enabled
		end,
		order = 40
	}

	return opts
end

-- enable plugin
function IHUD_Threat.prototype:Enable(core)
	IHUD_Threat.super.prototype.Enable(self, core)
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "Update")
	self:ScheduleRepeatingEvent(self.elementName, self.Update, 0.2, self)
	self:Update(self.unit)
end

-- disable plugin
function IHUD_Threat.prototype:Disable(core)
	IHUD_Threat.super.prototype.Disable(self, core)
	self:CancelScheduledEvent(self.elementName)
end

-- OVERRIDE
function IHUD_Threat.prototype:CreateFrame()
	IHUD_Threat.super.prototype.CreateFrame(self)
	
	self:CreateAggroBar()
end

-- create the aggro range indicator bar
function IHUD_Threat.prototype:CreateAggroBar()
	if not (self.aggroBar) then
		self.aggroBar = CreateFrame("StatusBar", nil, self.frame)
	end
	
	self.aggroBar:SetFrameStrata("BACKGROUND")
	self.aggroBar:SetWidth(self.settings.barWidth)
	self.aggroBar:SetHeight(self.settings.barHeight)
	
	if not (self.aggroBar.bar) then
		self.aggroBar.bar = self.aggroBar:CreateTexture(nil, "BACKGROUND")
	end
	
	self.aggroBar.bar:SetTexture(IceElement.TexturePath .. self.settings.barTexture .. "BG")
	self.aggroBar.bar:SetAllPoints(self.aggroBar)
	
	self.aggroBar:SetStatusBarTexture(self.aggroBar.bar)
	
	local r, g, b = self.settings.backgroundColor.r, self.settings.backgroundColor.g, self.settings.backgroundColor.b
	if (self.settings.backgroundToggle) then
		r, g, b = self:GetColor("CastCasting")
	end
	self.aggroBar:SetStatusBarColor(r, g, b, self.moduleSettings.lagAlpha)
	
	if (self.moduleSettings.side == IceCore.Side.Left) then
		self.aggroBar.bar:SetTexCoord(1, 0, 0, 0)
	else
		self.aggroBar.bar:SetTexCoord(0, 1, 0, 0)
	end
	
	self.aggroBar:ClearAllPoints()
	self.aggroBar:SetPoint("BOTTOM", self.frame, "BOTTOM", 0, 0)
end

-- bar stuff
function IHUD_Threat.prototype:Update(unit)

	-- only show bar if there's any threat
	if not Threat:IsActive() then return end

	IHUD_Threat.super.prototype.Update(self)

	if (unit and (unit ~= self.unit)) then
		return
	end

	if not unit then
		unit = self.unit
	end
	
	if not UnitExists("target") or not UnitCanAttack("player", "target") or UnitIsDead("target") or UnitIsFriend("player", "target") or UnitPlayerControlled("target")
	then
		self.frame:Hide()
		return
	else
		self.frame:Show()
	end

	-- get my threat and the tank threat
	local threatMe = Threat:GetThreat( UnitName("player") , UnitName("target") )
	local threatTank = 0
	local threatTankName = ""
	
	if UnitExists("targettarget") then
		threatTankName = UnitName("targettarget")
		threatTank = Threat:GetThreat( UnitName("targettarget"), UnitName("target") )
	else
		threatTank, threatTankName = Threat:GetMaxThreatOnTarget( UnitName("target") )
	end

	-- adjust max threat to avoid divide by 0
	if ( threatTank == 0 ) then
		threatTank = 1
	end

	-- aggro gain limit
	local threatMulti = 1.3
	if ( Threat:UnitInMeleeRange( "target" ) ) then
		threatMulti = 1.1
--	elseif ( UnitExists("targettarget") and ( UnitName("targettarget") == UnitName("player") ) ) then
--		threatMulti = 1
	elseif ( threatTankName == UnitName("player") ) then
		threatMulti = 1
	end
	
	-- get my threat percentage
	local threatPct = self:MathRound( (100/threatTank) * threatMe, 1 )

	Print( "threatMe = " .. threatMe .. ", threatTank = " .. threatTank .. ", threatPct = " .. threatPct )
	
	-- set percentage text
	self:SetBottomText1( threatPct .. "%" )
	self:SetBottomText2()

	-- set bar color
	if( threatMulti == 1 ) then
		self.color = "ThreatDanger"
	elseif( threatPct < 50 ) then
		self.color = "ThreatLow"
	elseif ( threatPct < 80 ) then
		self.color = "ThreatMedium"
	else
		self.color = "ThreatHigh"
	end

--[[	local g = floor( (255 / (threatTank*threatMulti) ) *threatMe )
	
	if(g>255) then g=255 end
	if(g<0) then g=0 end
	
	IceHUD.IceCore:SetColor("ThreatCustom", 255, 255, 0)	
	self.color = "ThreatCustom"
]]	
	-- set the bar value
	self:UpdateBar( threatMe / ( threatTank*threatMulti ), self.color )
	
	-- do the aggro indicator bar stuff, but only if it has changed
	if ( self.aggroBarMulti ~= threatMulti ) then
		self.aggroBarMulti = threatMulti

		local pos = 1 - (1 / threatMulti)
		local y = self.settings.barHeight - ( pos * self.settings.barHeight )
		
		if ( self.moduleSettings.side == IceCore.Side.Left ) then
			self.aggroBar.bar:SetTexCoord(1, 0, 0, pos)
		else
			self.aggroBar.bar:SetTexCoord(0, 1, 0, pos)
		end
		
		self.aggroBar:SetPoint("BOTTOM", self.frame, "BOTTOM", 0, y)
	end
end

-- rounding stuff
function IHUD_Threat.prototype:MathRound(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num  * mult + 0.5) / mult
end

-- Load us up
IceHUD.IHUD_Threat = IHUD_Threat:new()

-- overwrites the old IceHUD with a new one so the defaults of this Bar will be loaded
IceHUD:OnInitialize()
